﻿#region License
/* Copyright (C) 2009 Tim Coulter
 * 
 * This file is part of ExtremeML.
 * 
 * ExtremeML is free software: you can redistribute
 * it and/or modify it under the terms of the GNU General Public
 * License, as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See
 * the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see http://www.gnu.org/licenses/.
*/
#endregion

using System;
using System.Collections.Generic;
using DocumentFormat.OpenXml;
using DocumentFormat.OpenXml.Spreadsheet;
using ExtremeML.Core;
using ExtremeML.Packaging;
using ExtremeML.Spreadsheet.Address;

namespace ExtremeML.Spreadsheet
{
	// This is a partial class that adds custom functionality to its auto-generated partner.
	public sealed partial class ConditionalFormattingWrapper
	{
		#region Custom Properties

		/// <summary>
		/// Gets the containing <see cref="WorksheetPartWrapper"/>.
		/// </summary>
		public WorksheetPartWrapper WorksheetPart
		{
			get { return (WorksheetPartWrapper)Parent.Parent.Parent; }
		}

		/// <summary>
		/// Gets the <see cref="CellAreaCollection"/> that manages targeted cell addressing for the wrapped instance.
		/// </summary>
		public CellAreaCollection CellAreas { get; private set; }

		#endregion
		#region Custom Methods


		#endregion
		#region Overrides

		/// <summary>
		/// Implements custom initialization functionality.
		/// </summary>
		protected override void OnCustomInitializing()
		{
			CellAreas = new CellAreaCollection(true);
		}

		/// <summary>
		/// Implements custom population functionality.
		/// </summary>
		protected override void OnCustomDecomposing()
		{
			CellAreas = new CellAreaCollection(Wrapped.SequenceOfReferences, true);
		}

		/// <summary>
		/// Establishes relationships with other package content objects.
		/// </summary>
		protected override void OnCustomActivating()
		{
			RegisterAddressDependencies();
		}

		/// <summary>
		/// Implements custom persistence functionality.
		/// </summary>
		protected override void OnCustomRecomposing(
			ConditionalFormatting wrapped)
		{
			Wrapped.SequenceOfReferences.Items.Clear();

			foreach (var area in CellAreas)
			{
				Wrapped.SequenceOfReferences.Items.Add(new StringValue(area.Range));
			}
		}

		/// <summary>
		/// Implements custom deletion functionality.
		/// </summary>
		protected override void OnCustomRemoved()
		{
			ReleaseAddressDependencies();
		}

		/// <summary>
		/// Returns the set of <see cref="IWrapper"/> instances defined as custom properties.
		/// </summary>
		protected override IEnumerable<IWrapper> GetCustomWrapperMembers()
		{
			foreach (var area in CellAreas)
			{
				yield return area;
			}
		}

		#endregion
		#region Implementation Methods

		/// <summary>
		/// Registers addressable contained objects for notification of address changes.
		/// </summary>
		private void RegisterAddressDependencies()
		{
			foreach (var area in CellAreas)
			{
				RegisterAddressDependencies(area);
			}
		}

		/// <summary>
		/// Registers the passed <see cref="CellArea"/> for notification of address changes.
		/// </summary>
		private void RegisterAddressDependencies(
			CellArea area)
		{
			WorksheetPart.AddressDependencies.Add(area);

			area.RangeDeleted += HandleRangeDeletion;
		}

		/// <summary>
		/// Releases previously registered contained objects from notification of address changes.
		/// </summary>
		private void ReleaseAddressDependencies()
		{
			foreach (var area in CellAreas)
			{
				ReleaseAddressDependencies(area);
			}
		}

		/// <summary>
		/// Releases the passed <see cref="CellArea"/> from notification of address changes.
		/// </summary>
		private void ReleaseAddressDependencies(
			CellArea area)
		{
			WorksheetPart.AddressDependencies.Remove(area);

			area.RangeDeleted -= HandleRangeDeletion;
		}

		/// <summary>
		/// Handles the range deletion event of <see cref="CellArea"/>.
		/// </summary>
		private void HandleRangeDeletion(
			object sender,
			EventArgs e)
		{
			ReleaseAddressDependencies(sender as CellArea);

			if (CellAreas.Count == 0)
			{
				Remove();
			}
		}

		#endregion
		#region Static Methods


		#endregion
	}
}
