﻿using System;
using System.Collections.Generic;
using DocumentFormat.OpenXml.Spreadsheet;
using ExtremeML.Core;
using ExtremeML.Spreadsheet.Address;

namespace ExtremeML.Spreadsheet
{
	// This is a partial class that adds custom functionality to its auto-generated partner.
	public sealed partial class RowWrapper
	{
		#region Custom Properties

		/// <summary>
		/// Gets the containing <see cref="WorksheetWrapper"/>.
		/// </summary>
		public WorksheetWrapper Worksheet
		{
			get { return ((WorksheetWrapper)Parent.Parent.Parent); }
		}

		/// <summary>
		/// Gets the <see cref="RowSpan"/> that manages row addressing for the wrapped row.
		/// </summary>
		public RowSpan RowSpan { get; private set; }

		#endregion
		#region Custom Methods

		/// <summary>
		/// Sets the content of the cell at the specified column position to the passed value.
		/// </summary>
		/// <param name="index">The zero-based index of the target cell.</param>
		/// <param name="value">The value to be assigned to the cell.</param>
		public void SetCellValue(
			int index,
			object value)
		{
			if (index < 0) throw new ArgumentOutOfRangeException("index");

			GetCell(index, true).SetValue(value);
		}

		/// <summary>
		/// Returns the value of the cell at the specified column position.
		/// </summary>
		/// <typeparam name="T">One of the supported data types (see <see cref="CellWrapper"/>).</typeparam>
		/// <param name="index">The zero-based index of the target cell.</param>
		/// <returns>The value of the specified cell.</returns>
        public T GetCellValue<T>(
			int index)
		{
			if (index < 0) throw new ArgumentOutOfRangeException("index");

			var cell = GetCell(index, false);
			return cell == null ? default(T) : cell.GetValue<T>();
		}

		/// <summary>
		/// Sets the formula of the cell at the specified column position to the passed value.
		/// </summary>
		/// <param name="index">The zero-based index of the target cell.</param>
		/// <param name="value">The formula to be assigned to the cell.</param>
        public void SetCellFormula(
			int index,
			string value)
		{
			if (index < 0) throw new ArgumentOutOfRangeException("index");

			GetCell(index, true).SetFormula(value);
		}

		/// <summary>
		/// Returns the formula of the cell at the specified column position.
		/// </summary>
		/// <param name="index">The zero-based index of the target cell.</param>
		/// <returns>The formula assigned to the cell.</returns>
        public string GetCellFormula(
			int index)
		{
			if (index < 0) throw new ArgumentOutOfRangeException("index");

			var cell = GetCell(index, false);
			return cell == null ? null : cell.GetFormula();
		}

		/// <summary>
		/// Returns the <see cref="CellWrapper"/> that represents the worksheet cell at the
		/// specified coordinates, optionally creating it if it does not already exist.
		/// </summary>
		/// <param name="index">The zero-based index of the the target cell.</param>
		/// <param name="force">A value that indicates whether the cell wrapper should be created if it does not exist.</param>
		/// <returns>A <see cref="CellWrapper"/> that represents the specified worksheet cell.</returns>
		public CellWrapper GetCell(
			int index,
			bool force)
		{
			if (index < 0) throw new ArgumentOutOfRangeException("index");

			if (Cells.Contains(index))
			{
				return Cells.Find(index);
			}
			
			if (force)
			{
				return Cells.Add(index);
			}

			return null;
		}

		#endregion
		#region Overrides

		/// <summary>
		/// Implements custom initialization functionality.
		/// </summary>
		protected override void OnCustomInitializing()
		{
			RowSpan = new RowSpan();
		}

		/// <summary>
		/// Implements custom population functionality.
		/// </summary>
		protected override void OnCustomDecomposing()
		{
			RowSpan = new RowSpan((int?)RowIndex - 1, (int?)RowIndex - 1);
		}

		/// <summary>
		/// Establishes relationships with other package content objects.
		/// </summary>
		protected override void OnCustomActivating()
		{
			RegisterAddressDependencies();
		}

		/// <summary>
		/// Implements custom persistence functionality.
		/// </summary>
		protected override void OnCustomRecomposing(
			Row wrapped)
		{
			RowIndex = (uint)RowSpan.Begin + 1;
		}

		/// <summary>
		/// Implements custom deletion functionality.
		/// </summary>
		protected override void OnCustomRemoved()
		{
			ReleaseAddressDependencies();
		}

		/// <summary>
		/// Returns the set of <see cref="IWrapper"/> instances defined as custom properties.
		/// </summary>
		protected override IEnumerable<IWrapper> GetCustomWrapperMembers()
		{
			yield return RowSpan;
		}

		#endregion
		#region Implementation Methods

		/// <summary>
		/// Registers addressable contained objects for notification of address changes.
		/// </summary>
		private void RegisterAddressDependencies()
		{
			Worksheet.WorksheetPart.AddressDependencies.Add(RowSpan);

			RowSpan.SpanDeleted += HandleSpanDeletion;
		}

		/// <summary>
		/// Releases previously registered contained objects from notification of address changes.
		/// </summary>
		private void ReleaseAddressDependencies()
		{
			if (Worksheet.WorksheetPart.AddressDependencies.Contains(RowSpan))
			{
				Worksheet.WorksheetPart.AddressDependencies.Remove(RowSpan);
			}

			RowSpan.SpanDeleted -= HandleSpanDeletion;
		}

		/// <summary>
		/// Handles the span deletion event of <see cref="RowSpan"/>.
		/// </summary>
		private void HandleSpanDeletion(
			object sender,
			EventArgs e)
		{
			Remove();
		}

		#endregion
		#region Static Methods


		#endregion
	}
}
